"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',module='.*OpenSSL.*')

from twisted.protocols.ftp import FTPFactory, FTP, AUTH_FAILURE
from twisted.internet import reactor
from ftplib import FTP as FFTP
from psutil import process_iter
from signal import SIGTERM
from logging import DEBUG, basicConfig, getLogger
from twisted.python import log as tlog
from tempfile import gettempdir,_get_candidate_names
from subprocess import Popen
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from pathlib import Path
from os import path
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QFTPServer():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking='',logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 21
		self.username = username or 'test'
		self.password = password or 'test'
		self.mocking = mocking or ''
		self.random_servers = ['ProFTPD 1.2.10','ProFTPD 1.3.4a','FileZilla ftp 0.9.43','Gene6 ftpd 3.10.0','FileZilla ftp 0.9.33','ProFTPD 1.2.8']
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))
		tlog.startLogging(open(temp_name, 'w'), setStdout=False)

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def ftp_server_main(self):
		_q_s = self

		class CustomFTPProtocol(FTP):

			def ftp_PASS(self, password):
				if self._user == _q_s.username and password == _q_s.password:
					_q_s.logs.info(['servers',{'server':'ftp_server','action':'login','status':'success','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':_q_s.username,'password':_q_s.password}])
				else:
					_q_s.logs.info(['servers',{'server':'ftp_server','action':'login','status':'failed','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':self._user,'password':password}])
				return AUTH_FAILURE

		class CustomFTPFactory(FTPFactory):
			protocol = CustomFTPProtocol
			portal = None
			
			def buildProtocol(self, address):
				p = self.protocol()
				p.portal = self.portal
				p.factory = self
				return p

		factory = CustomFTPFactory()
		reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
		reactor.run()

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'ftp_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'ftp_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'ftp_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'ftp_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'ftp_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.ftp_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False


	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			_ip = ip or self.ip
			_port = port or self.port 
			_username = username or self.username
			_password = password or self.password
			f = FFTP()
			f.connect(_ip,_port)
			f.login(_username,_password)
		except:
			pass

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'ftp_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		ftpserver = QFTPServer(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		ftpserver.run_server()


"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',module='.*OpenSSL.*')

from twisted.internet.protocol import Protocol,Factory
from twisted.internet import reactor
from psutil import process_iter
from signal import SIGTERM
from logging import DEBUG, basicConfig, getLogger
from twisted.python import log as tlog
from tempfile import gettempdir,_get_candidate_names
from struct import pack
from hashlib import sha1
from mysql.connector import connect as mysqlconnect
from subprocess import Popen
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from pathlib import Path
from pathlib import Path
from os import path
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QMysqlServer():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking=False,dict_=None,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 3306
		self.username = username or "test"
		self.password = password or "test"
		self.mocking = mocking or ''
		self.file_name = dict_ or None
		if not dict_:
			self.words = ["test"]
		else:
			self.load_words()
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))
		tlog.startLogging(open(temp_name, 'w'), setStdout=False)

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def load_words(self,):
		with open(self.file_name, 'r') as file:
			self.words = file.read().splitlines()

	def greeting(self):
		base = ['\x0a','5.7.00' + '\0','\x36\x00\x00\x00','12345678' + '\0','\xff\xf7','\x21','\x02\x00','\x0f\x81','\x15','\0' * 10,'123456789012' + '\0','mysql_native_password' + '\0']
		payload_len = list(pack('<I',len(''.join(base))))
		#payload_len[3] = '\x00'
		string_ = chr(payload_len[0]) + chr(payload_len[1]) + chr(payload_len[2]) + '\x00' + ''.join(base)
		return string_

	def too_many(self):
		base = ['\xff','\x10\x04','#08004','Too many connections']
		payload_len = list(pack('<I',len(''.join(base))))
		#payload_len[3] = '\x02'
		string_ = chr(payload_len[0]) + chr(payload_len[1]) + chr(payload_len[2]) + '\x02' + ''.join(base)
		return string_

	def parse_data(self,data):
		username,password = '',''
		try:
			username_len = data[36:].find(b'\x00')
			username = data[36:].split(b'\x00')[0]
			password_len = ord(data[36+username_len+1])
			password = data[36+username_len+2:36+username_len+2+password_len]
			rest_ = data[36+username_len+2+password_len:]
			if b'mysql_native_password' in rest_:
				if len(password) == 20:
					return username,password,True
		except:
			pass
		return username,password,False

	def decode(self,hash):
		try:
			for word in self.words:
				temp = word
				word = word.strip('\n')
				hash1 = sha1(word).digest()
				hash2 = sha1(hash1).digest()
				encrypted = ''.join(chr(ord(a) ^ ord(b)) for a,b in zip(hash1,sha1('12345678123456789012' + hash2).digest()))
				if encrypted == hash:
					return temp
		except:
			pass

		return None

	def mysql_server_main(self):
		_q_s = self

		class CustomMysqlProtocol(Protocol):

			_state = None

			def connectionMade(self):
				self._state = 1
				self.transport.write(_q_s.greeting().encode())
				_q_s.logs.info(["servers",{'server':'mysql_server','action':'connection','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port}])

			def dataReceived(self, data):
				if self._state == 111111111111:
					username, password, good = _q_s.parse_data(data)
					if good:
						if password:
							_x = _q_s.decode(password)
							if _x == _q_s.password and _x != None:
								_q_s.logs.info(["servers",{'server':'mysql_server','action':'login','status':'success','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':_q_s.username,'password':_q_s.password}])
							else:
								_q_s.logs.info(["servers",{'server':'mysql_server','action':'login','status':'failed','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':username,'password':password}])
						else:
							_q_s.logs.info(["servers",{'server':'mysql_server','action':'login','status':'failed','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':'UnKnown','password':''.join(hex(ord(c))[2:] for c in data)}])

					self.transport.write(_q_s.too_many().encode())
				else:
					self.transport.loseConnection()

			def connectionLost(self, reason):
				self._state = None

		factory = Factory()
		factory.protocol = CustomMysqlProtocol
		reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
		reactor.run()

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'mysql_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'mysql_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'mysql_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'mysql_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'mysql_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.mysql_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			_ip = ip or self.ip
			_port = port or self.port 
			_username = username or self.username
			_password = password or self.password
			cnx = mysqlconnect(user=_username, password=_password, host=_ip, port=_port,database='test',connect_timeout=1000)
		except:
			pass


	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'mysql_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qmysqlserver = QMysqlServer(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		qmysqlserver.run_server()



"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',module='.*paramiko.*')

from paramiko import ServerInterface,Transport,RSAKey,AutoAddPolicy
from socket import socket,AF_INET,SOCK_STREAM,SOL_SOCKET,SO_REUSEADDR
from _thread import start_new_thread
from io import StringIO
from random import choice
from multiprocessing import Process
from psutil import process_iter
from signal import SIGTERM
from time import sleep
from paramiko import SSHClient
from logging import DEBUG, basicConfig, getLogger
from tempfile import gettempdir,_get_candidate_names
from subprocess import Popen
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from email.parser import BytesParser
from pathlib import Path
from os import path
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QSSHServer():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking=False,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 22
		self.username = username or "test"
		self.password = password or "test"
		self.mocking = mocking or ''
		self.random_servers = ['OpenSSH 7.5','OpenSSH 7.3','Serv-U SSH Server 15.1.1.108','OpenSSH 6.4']
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))

	def generate_pub_pri_keys(self):
		try:
			key = RSAKey.generate(2048)
			string_io = StringIO()
			key.write_private_key(string_io)
			return key.get_base64(), string_io.getvalue()
		except:
			pass
		return None,None

	def ssh_server_main(self):
		_q_s = self

		class SSHHandle(ServerInterface):

			def __init__(self,ip,port):
				self.ip = ip
				self.port = port
				ServerInterface.__init__(self)

			def check_auth_password(self, username, password):
				username, password = username.encode("utf-8"), password.encode("utf-8")
				if username ==  _q_s.username and password == _q_s.password:
					_q_s.logs.info(["servers",{'server':'ssh_server','action':'login','status':'success','ip':self.ip,'port':self.port,'username':username,'password':password}])
				else:
					_q_s.logs.info(["servers",{'server':'ssh_server','action':'login','status':'failed','ip':self.ip,'port':self.port,'username':username,'password':password}])

		def ConnectionHandle(client,priv):
			try:
				t = Transport(client)
				ip, port = client.getpeername()
				_q_s.logs.info(["servers",{'server':'ssh_server','action':'connection','ip':ip,'port':port}])
				t.local_version = 'SSH-2.0-'+choice(self.random_servers)
				t.add_server_key(RSAKey(file_obj=StringIO(priv)))
				t.start_server(server=SSHHandle(ip,port))
				chan = t.accept(1)
				if not chan is None:
					chan.close()
			except:
				pass

		sock = socket(AF_INET,SOCK_STREAM)
		sock.setsockopt(SOL_SOCKET,SO_REUSEADDR, 1)
		sock.bind((self.ip, self.port))
		sock.listen(1)
		pub, priv = self.generate_pub_pri_keys()
		while True:
			try:
				client, addr = sock.accept()
				start_new_thread(ConnectionHandle,(client,priv,))
			except:
				pass

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'ssh_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'ssh_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'ssh_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'ssh_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'ssh_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.ssh_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			sleep(2)
			_ip = ip or self.ip
			_port = port or self.port 
			_username = username or self.username
			_password = password or self.password
			ssh = SSHClient()
			ssh.set_missing_host_key_policy(AutoAddPolicy()) #if you have default ones, remove them before using this.. 
			ssh.connect(_ip, port=_port,username=_username,password=_password)
		except:
			pass

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'ssh_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qsshserver = QSSHServer(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		qsshserver.run_server()


"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',module='.*OpenSSL.*')

from dns.resolver import query as dsnquery
from twisted.internet import reactor
from twisted.internet.protocol import Protocol,ClientFactory,Factory
from psutil import process_iter
from signal import SIGTERM
from requests import get
from logging import DEBUG, basicConfig, getLogger
from twisted.python import log as tlog
from tempfile import gettempdir,_get_candidate_names
from subprocess import Popen
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from email.parser import BytesParser
from pathlib import Path
from os import path
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QHTTPProxyServer():
	def __init__(self,ip=None,port=None,mocking=None,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 8080 
		self.mocking = mocking or ''
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))
		tlog.startLogging(open(temp_name, 'w'), setStdout=False)

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def http_proxy_server_main(self):
		_q_s = self

		class CustomProtocolParent(Protocol):

			def __init__(self):
				self.buffer = None
				self.client = None

			def resolve_domain(self,request_string):
				try:
					_, parsed_request = request_string.split(b'\r\n', 1)
					headers = BytesParser().parsebytes(parsed_request)
					host = headers["host"].split(":")
					_q_s.logs.info(["servers",{'server':'http_proxy_server','action':'query','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'payload':host[0]}])
					#return "127.0.0.1"
					return dsnquery(host[0], 'A')[0].address
				except Exception as e:
					_q_s.logs.error(["errors",{'server':'http_proxy_server','error':'resolve_domain',"type":"error -> "+repr(e)}])
				return None

			def dataReceived(self, data):
				_q_s.logs.info(["servers",{'server':'http_proxy_server','action':'connection','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port}])
				try:
					ip = self.resolve_domain(data)
					if ip:
						factory = ClientFactory()
						factory.CustomProtocolParent_= self
						factory.protocol = CustomProtocolChild
						reactor.connectTCP(ip, 80, factory)
					else:
						self.transport.loseConnection()

					if self.client:
						self.client.write(data)
					else:
						self.buffer = data
				except:
					pass

			def write(self, data):
				self.transport.write(data)

		class CustomProtocolChild(Protocol):
			def connectionMade(self):
				self.write(self.factory.CustomProtocolParent_.buffer)

			def dataReceived(self, data):
				self.factory.CustomProtocolParent_.write(data)

			def write(self, data):
				self.transport.write(data)

		factory = Factory()
		factory.protocol = CustomProtocolParent
		reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
		reactor.run()

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'http_proxy_server','action':'process','status':'success','ip':self.ip,'port':self.port}])
					else:
						self.logs.info(["servers",{'server':'http_proxy_server','action':'process','status':'error','ip':self.ip,'port':self.port}])
				else:
					self.logs.info(["servers",{'server':'http_proxy_server','action':'setup','status':'error','ip':self.ip,'port':self.port}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'http_proxy_server','action':'process','status':'success','ip':self.ip,'port':self.port}])
				else:
					self.logs.info(["servers",{'server':'http_proxy_server','action':'process','status':'error','ip':self.ip,'port':self.port}])
		else:
			self.http_proxy_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,domain=None):
		try:
			_ip = ip or self.ip
			_port = port or self.port
			_domain = domain or "http://yahoo.com"
			get(_domain, proxies={"http":'http://{}:{}'.format(_ip,_port)}).text.encode('ascii', 'ignore')
		except:
			pass

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'http_proxy_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qhttpproxyserver = QHTTPProxyServer(ip=parsed.ip,port=parsed.port,mocking=parsed.mocking,logs=parsed.logs)
		qhttpproxyserver.run_server()


"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from psutil import process_iter
from signal import SIGTERM
from argparse import ArgumentParser
from os import scandir
from socket import socket, AF_INET, SOCK_STREAM
from time import sleep
from json import JSONEncoder, dumps
from collections import Mapping
from logging import Handler
from sys import stdout
from pygments import highlight, lexers, formatters
from datetime import datetime

def clean_all():
	for entry in scandir('.'):
		if entry.is_file() and entry.name.endswith("_server.py"):
			kill_servers(entry.name)

def kill_servers(name):
	try:
		for process in process_iter():
			cmdline = ' '.join(process.cmdline())
			if '--custom' in cmdline and name in cmdline:
				process.send_signal(SIGTERM)
				process.kill()
		if self.process != None:
			self.process.kill()
	except:
		pass

def get_free_port():
	port = 0
	try:
		tcp = socket(AF_INET, SOCK_STREAM)
		tcp.bind(('', 0))
		addr, port = tcp.getsockname()
		tcp.close()
	except:
		pass
	return port

def server_arguments():
	_server_parser = ArgumentParser(prog="Server")
	_server_parsergroupdeq = _server_parser.add_argument_group('Initialize Server')
	_server_parsergroupdeq.add_argument('--ip',type=str, help="Change server ip, current is 0.0.0.0", required=False,metavar='')
	_server_parsergroupdeq.add_argument('--port',type=int, help="Change port", required=False,metavar='')
	_server_parsergroupdeq.add_argument('--username',type=str, help="Change username", required=False,metavar='')
	_server_parsergroupdeq.add_argument('--password',type=str, help="Change password", required=False,metavar='')
	_server_parsergroupdeq.add_argument('--resolver_addresses',type=str, help="Change resolver address", required=False,metavar='')
	_server_parsergroupdeq.add_argument('--domain',type=str, help="A domain to test", required=False,metavar='')
	_server_parsergroupdeq.add_argument('--mocking',type=str, help="Random banner", required=False)
	_server_parsergroupdes = _server_parser.add_argument_group('Sinffer options')
	_server_parsergroupdes.add_argument('--filter',type=str, help="setup the Sinffer filter", required=False)
	_server_parsergroupdes.add_argument('--interface',type=str, help="sinffer interface E.g eth0", required=False)
	_server_parsergroupdef = _server_parser.add_argument_group('Initialize Loging')
	_server_parsergroupdef.add_argument('--logs',type=str, help="db, terminal or all", required=False, default="terminal")
	_server_parsergroupdea = _server_parser.add_argument_group('Auto Configuration')
	_server_parsergroupdea.add_argument('--docker',action='store_true', help="Run project in docker", required=False)
	_server_parsergroupdea.add_argument('--aws',action='store_true', help="Run project in aws", required=False)
	_server_parsergroupdea.add_argument('--test',action='store_true', help="Test current server", required=False)
	_server_parsergroupdea.add_argument('--custom',action='store_true', help="Run custom server", required=False)
	_server_parsergroupdea.add_argument('--auto',action='store_true', help="Run auto configured with random port", required=False)
	return _server_parser.parse_args()

class ComplexEncoder(JSONEncoder):
	def default(self, obj):
		if isinstance(obj, bytes):
			return obj.decode()
		else:
			return repr(obj)
		return JSONEncoder.default(self, obj)

class CustomHandler(Handler):
	def __init__(self):
		Handler.__init__(self)

	def emit(self, record):
		try:
			time_now = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
			if record.msg[0] == "servers":
				if "server" in record.msg[1]:
					temp = record.msg[1]
					action = record.msg[1]['action']
					server = temp['server'].replace('server','').replace('_','')
					del temp['server']
					del temp['action']
					stdout.write("[{}] [{}] [{}] -> {}\n".format(time_now,server,action,dumps(temp, sort_keys=True, cls=ComplexEncoder)))
			else:
				stdout.write(dumps(record.msg, sort_keys=True,cls=ComplexEncoder))
		except Exception as e:
			stdout.write(dumps({"logger":repr(record)}, sort_keys=True,cls=ComplexEncoder))
		stdout.flush()

	def emit_old(self, record):
		try:
			if record.msg[0] == "servers":
				if "server" in record.msg[1]:
					temp = record.msg[1]
					server = temp['server']
					del temp['server']
					stdout.write(highlight(dumps({server:temp}, sort_keys=True,indent=4,cls=ComplexEncoder), lexers.JsonLexer(), formatters.TerminalFormatter()))
			else:
				stdout.write(highlight(dumps(record.msg, sort_keys=True,indent=4,cls=ComplexEncoder), lexers.JsonLexer(), formatters.TerminalFormatter()))
		except Exception as e:
			stdout.write(highlight(dumps({"logger":repr(record)}, sort_keys=True,indent=4,cls=ComplexEncoder), lexers.JsonLexer(), formatters.TerminalFormatter()))
		stdout.flush()


"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',module='.*OpenSSL.*')

from cgi import FieldStorage
from psutil import process_iter
from signal import SIGTERM
from requests import get,post
from requests.packages.urllib3 import disable_warnings
from time import sleep
from logging import DEBUG, basicConfig, getLogger
from twisted.internet import reactor
from twisted.web.server import Site
from twisted.web.resource import Resource
from random import choice
from twisted.python import log as tlog
from tempfile import gettempdir,_get_candidate_names
from subprocess import Popen
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from pathlib import Path
from os import path
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

disable_warnings()

class QHTTPServer():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking=False,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 80
		self.username = username or "test"
		self.password = password or "test"
		self.mocking = mocking or ''
		self.key = path.join(gettempdir(), next(_get_candidate_names()))
		self.cert = path.join(gettempdir(), next(_get_candidate_names()))
		self.random_servers = ['Apache','nginx','Microsoft-IIS/7.5','Microsoft-HTTPAPI/2.0','Apache/2.2.15','SmartXFilter','Microsoft-IIS/8.5','Apache/2.4.6','Apache-Coyote/1.1','Microsoft-IIS/7.0','Apache/2.4.18','AkamaiGHost','Apache/2.2.25','Microsoft-IIS/10.0','Apache/2.2.3','nginx/1.12.1','Apache/2.4.29','cloudflare','Apache/2.2.22']
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))
		tlog.startLogging(open(temp_name, 'w'), setStdout=False)

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def http_server_main(self):
		_q_s = self

		class MainResource(Resource):

			isLeaf = True

			home_file = b'''
<!DOCTYPE html>
<html>
   <head>
	  <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.0.0-beta.3/css/bootstrap.min.css" />
	  <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" />
	  <meta http-equiv="content-type" content="text/html;charset=utf-8" />
	  <title>Login</title>
	  <style>
		 body,html{height: 100%;text-align: center;},
	  </style>
   </head>
   <body>
	  <div class="container-fluid h-100">
		 <div class="row justify-content-center h-100 align-items-center">
			<div class="col col-xl-3">
			   <b>We'll back soon..</b> 
			</div>
		 </div>
	  </div>
   </body>
</html>'''

			login_file = b'''<!DOCTYPE html>
<html>
   <head>
	  <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.0.0-beta.3/css/bootstrap.min.css" />
	  <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" />
	  <meta http-equiv="content-type" content="text/html;charset=utf-8" />
	  <title>Login</title>
	  <style>body,html {height: 100%;}</style>
   </head>
   <body>
	  <div class="container-fluid h-100">
		 <div class="row justify-content-center h-100 align-items-center">
			<div class="col col-xl-3">
			   <form id="login" action="" method="post">
				  <div class="form-group">
					 <input class="form-control form-control-sm" name="username" type="text" placeholder="username" id="username">
				  </div>
				  <div class="form-group">
					 <input class="form-control form-control-sm" name="password" type="password" placeholder="password" id="password">
				  </div>
				  <div class="form-group">
					 <button class="btn btn-default btn-sm btn-block" type="submit">login</button>
				  </div>
			   </form>
			</div>
		 </div>
	  </div>
   </body>
</html>
'''
			server = ""


			if isinstance(_q_s.mocking, bool):
				if _q_s.mocking == True:
					server = choice(_q_s.random_servers)
			elif isinstance(_q_s.mocking, str):
				server = _q_s.mocking

			def render(self, request):
				_q_s.logs.info(["servers",{'server':'http_server','action':'connection','ip':request.getClientIP()}])
				if self.server != "":
					request.responseHeaders.removeHeader("Server")
					request.responseHeaders.addRawHeader("Server", self.server)

				if request.method == b"GET":
					_q_s.logs.info(["servers",{'server':'http_server','action':'get','ip':request.getClientIP()}])
					if request.uri == "/login.html":
						if _q_s.username != '' and _q_s.password != '':
							request.responseHeaders.addRawHeader("Content-Type", "text/html; charset=utf-8")
							return self.login_file

					request.responseHeaders.addRawHeader("Content-Type", "text/html; charset=utf-8")
					return self.home_file

				elif request.method == b"POST":
					self.headers = request.getAllHeaders()
					_q_s.logs.info(["servers",{'server':'http_server','action':'post','ip':request.getClientIP()}])
					if _q_s.username != '' and _q_s.password != '':
						form = FieldStorage(fp=request.content,headers=self.headers,environ={'REQUEST_METHOD':'POST','CONTENT_TYPE':self.headers[b'content-type'],})
						if 'username' in form and 'password' in form:
							if form['username'].value == _q_s.username and form['password'].value == _q_s.password:
								_q_s.logs.info(["servers",{'server':'http_server','action':'login','status':'success','ip':request.getClientIP(),'username':_q_s.username,'password':_q_s.password}])
							else:
								_q_s.logs.info(["servers",{'server':'http_server','action':'login','status':'failed','ip':request.getClientIP(),'username':form['username'].value,'password':form['password'].value}])

					request.responseHeaders.addRawHeader("Content-Type", "text/html; charset=utf-8")
					return self.home_file
				else:
					request.responseHeaders.addRawHeader("Content-Type", "text/html; charset=utf-8")
					return self.home_file

		reactor.listenTCP(self.port, Site(MainResource()))
		reactor.run()

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'http_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'http_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'http_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'http_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'http_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.http_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			sleep(2)
			_ip = ip or self.ip
			_port = port or self.port
			_username = username or self.username
			_password = password or self.password
			get('http://{}:{}'.format(_ip,_port),verify=False)
			post('http://{}:{}'.format(_ip,_port),data={'username': (None, _username),'password': (None, _password)})
		except:
			pass

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'http_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qhttpserver = QHTTPServer(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		qhttpserver.run_server()


"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from twisted.mail.imap4 import IMAP4Server
from twisted.internet.protocol import Factory
from twisted.internet import reactor
from random import choice
from psutil import process_iter
from signal import SIGTERM
from logging import DEBUG, basicConfig, getLogger
from twisted import cred
from imaplib import IMAP4
from twisted.python import log as tlog
from tempfile import gettempdir,_get_candidate_names
from subprocess import Popen
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from pathlib import Path
from os import path
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QIMAPServer():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking=False,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 143
		self.username = username or "test"
		self.password = password or "test"
		self.mocking = mocking or ''
		self.random_servers = [b'OK Microsoft Exchange Server 2003 IMAP4rev1 server version 6.5.6944.0 DC9 ready']
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		#self.disable_logger()

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))
		tlog.startLogging(open(temp_name, 'w'), setStdout=False)

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def imap_server_main(self):

		_q_s = self

		class CustomIMAP4Server(IMAP4Server):

			def connectionMade(self):
				_q_s.logs.info(["servers",{'server':'imap_server','action':'connection','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port}])

				if isinstance(_q_s.mocking, bool):
					if _q_s.mocking == True:
						self.sendPositiveResponse(message=choice(_q_s.random_servers))
				elif isinstance(_q_s.mocking, str):
					self.sendPositiveResponse(message=choice(_q_s.random_servers))
				else:
					self.sendPositiveResponse(message=b'Welcome')

			def authenticateLogin(self, user, passwd):
				if user == _q_s.username and passwd == _q_s.password:
					_q_s.logs.info(["servers",{'server':'imap_server','action':'login','status':'success','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':_q_s.username,'password':_q_s.password}])
				else:
					_q_s.logs.info(["servers",{'server':'imap_server','action':'login','status':'failed','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':user,'password':passwd}])
				
				raise cred.error.UnauthorizedLogin()

			def lineReceived(self, line):
				try:
					_line = line.split(b" ")[1]
					if _line.lower().startswith(b"login") or _line.lower().startswith(b"capability"):
						IMAP4Server.lineReceived(self, line)
				except:
					pass

		class CustomIMAPFactory(Factory):
			protocol = CustomIMAP4Server
			portal = None
			
			def buildProtocol(self, address):
				p = self.protocol()
				p.portal = self.portal
				p.factory = self
				return p

		factory = CustomIMAPFactory()
		reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
		reactor.run()

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'imap_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'imap_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'imap_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'imap_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'imap_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.imap_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			_ip = ip or self.ip
			_port = port or self.port 
			_username = username or self.username
			_password = password or self.password
			imap_test = IMAP4(_ip,_port)
			imap_test.login(_username, _password)
		except:
			pass

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'imap_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qimapserver = QIMAPServer(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		qimapserver.run_server()


"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',category=DeprecationWarning)

from warnings import filterwarnings
filterwarnings(action='ignore',module='.*impacket.*')

from logging import StreamHandler,getLogger,DEBUG
from impacket import smbserver
from impacket.smbconnection import SMBConnection
from tempfile import mkdtemp
from shutil import rmtree
from impacket.ntlm import compute_lmhash, compute_nthash
from multiprocessing import Process
from psutil import process_iter
from signal import SIGTERM
from time import sleep
from logging import DEBUG, basicConfig, getLogger
from pathlib import Path
from os import path
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from subprocess import Popen
from tempfile import gettempdir,_get_candidate_names
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

#loggers = [logging.getLogger(name) for name in logging.root.manager.loggerDict]
#print([logging.getLogger(name) for name in logging.root.manager.loggerDict])

class QSMBServer():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking=False,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 445
		self.username = username or "test"
		self.password = password or "test"
		self.mocking = mocking or ''
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def disable_logger(self):
		getLogger('impacket').propagate = False

	def smb_server_main(self):
		_q_s = self

		class Logger(object):
			def write(self, message):
				#sys.stdout.write(str(">>>>" + message))
				#sys.stdout.flush()
				try:
					if "Incoming connection" in message.strip() or "AUTHENTICATE_MESSAGE" in message.strip() or "authenticated successfully" in message.strip():
						_q_s.logs.info(["servers",{'server':'smb_server','action':'connection','msg':message.strip()}])
					elif ":4141414141414141:" in message.strip():
						parsed = message.strip().split(":")
						if len(parsed)>2:
							_q_s.logs.info(["servers",{'server':'smb_server','action':'login','workstation':parsed[0],'test':parsed[1]}])
				except Exception as e:
					_q_s.logs.error(["errors",{'server':'smb_server','error':'write',"type":"error -> "+repr(e)}])

		handler = StreamHandler(Logger())
		getLogger("impacket").addHandler(handler)
		getLogger("impacket").setLevel(DEBUG)

		dirpath = mkdtemp()
		server = smbserver.SimpleSMBServer(listenAddress=self.ip, listenPort=self.port)
		#server.removeShare("IPC$")
		server.addShare('C$', dirpath, '',readOnly='yes')
		server.setSMB2Support(True)
		server.addCredential(self.username, 0, compute_lmhash(self.password), compute_nthash(self.password))
		server.setSMBChallenge('')
		server.start()
		rmtree(dirpath)

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'smb_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'smb_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'smb_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'smb_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'smb_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.smb_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			sleep(2)
			_ip = ip or self.ip
			_port = port or self.port
			_username = username or self.username
			_password = password or self.password
			smb_client = SMBConnection(_ip, _ip,sess_port=_port)
			smb_client.login(_username, _password)
		except Exception as e:
			self.logs.error(["errors",{'server':'smb_server','error':'write',"type":"error -> "+repr(e)}])

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'smb_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':

	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qsmbserver = QSMBServer(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		qsmbserver.run_server()



"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from multiprocessing import Process
from psutil import process_iter
from signal import SIGTERM
from time import sleep
from logging import DEBUG, basicConfig, getLogger
from socketserver import TCPServer, StreamRequestHandler, ThreadingMixIn
from struct import unpack
from requests import get
from pathlib import Path
from os import path
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from subprocess import Popen
from tempfile import gettempdir,_get_candidate_names
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QSOCKS5Server():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking=False,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 1080
		self.username = username or "test"
		self.password = password or "test"
		self.mocking = mocking or ''
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))

	def socks5_server_main(self):
		_q_s = self

		class CustomStreamRequestHandler(StreamRequestHandler):
			def handle(self):
				_q_s.logs.info(["servers",{'server':'socks5_server','action':'connection','ip':self.client_address[0],'port':self.client_address[1]}])
				v,m = unpack("!BB", self.connection.recv(2))
				if v == 5:
					if 2 in unpack("!"+"B"*m,self.connection.recv(m)):
						self.connection.sendall(b'\x05\x02')
						if 1 in unpack("B",self.connection.recv(1)):
							_len = ord(self.connection.recv(1))
							username = self.connection.recv(_len)
							_len = ord(self.connection.recv(1))
							password = self.connection.recv(_len)
							if username == _q_s.username and password == _q_s.password:
								_q_s.logs.info(["servers",{'server':'socks5_server','action':'login','status':'success','ip':self.client_address[0],'port':self.client_address[1],'username':_q_s.username,'password':_q_s.password}])
							else:
								_q_s.logs.info(["servers",{'server':'socks5_server','action':'login','status':'failed','ip':self.client_address[0],'port':self.client_address[1],'username':username,'password':password}])
				self.server.close_request(self.request)

		class ThreadingTCPServer(ThreadingMixIn, TCPServer):
			pass

		TCPServer.allow_reuse_address = True
		server = ThreadingTCPServer((self.ip, self.port), CustomStreamRequestHandler)
		server.serve_forever()

	def run_server(self,process=False,auto=False):
		if process:
			if self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
		else:
			self.socks5_server_main()

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'socks5_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'socks5_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'socks5_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'socks5_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'socks5_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.socks5_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			sleep(2)
			_ip = ip or self.ip
			_port = port or self.port 
			_username = username or self.username
			_password = password or self.password
			get('https://yahoo.com', proxies=dict(http='socks5://{}:{}@{}:{}'.format(_username,_password,_ip,_port),https='socks5://{}:{}@{}:{}'.format(_username,_password,_ip,_port)))
		except:
			pass
			
	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'socks5_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		QSOCKS5Server = QSOCKS5Server(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		QSOCKS5Server.run_server()


"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',module='.*OpenSSL.*')

from twisted.internet.protocol import Protocol,Factory
from twisted.internet import reactor
from psutil import process_iter
from signal import SIGTERM
from Crypto.Cipher import DES
from binascii import unhexlify
from logging import DEBUG, basicConfig, getLogger
from logging import DEBUG, basicConfig, getLogger
from twisted.python import log as tlog
from tempfile import gettempdir,_get_candidate_names
from subprocess import Popen
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from pathlib import Path
from os import path
from vncdotool import api as vncapi
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QVNCServer():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking=False,dict_=None,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 5900
		self.username = username or "test"
		self.password = password or "test"
		self.mocking = mocking or ''
		self.random_servers = ['VNC Server']
		self.file_name = dict_ or None
		self.challenge = unhexlify("00000000901234567890123456789012")
		if not dict_:
			self.words = ["test"]
		else:
			self.load_words()
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		#self.disable_logger()

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))
		tlog.startLogging(open(temp_name, 'w'), setStdout=False)

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def load_words(self,):
		with open(self.file_name, 'r') as file:
			self.words = file.read().splitlines()

	def decode(self,c, r):
		try:
			for word in self.words:
				temp = word
				word = word.strip('\n').ljust(8, '\00')[:8]
				rev_word = []
				for i in range(0,8):
					rev_word.append(chr(int('{:08b}'.format(ord(word[i]))[::-1], 2)))
				output = DES.new(''.join(rev_word).encode('utf-8'),DES.MODE_ECB).encrypt(c)
				if output == r:
					return temp
		except:
			pass

		return None

	def vnc_server_main(self):
		_q_s = self

		class CustomVNCProtocol(Protocol):

			_state = None

			def connectionMade(self):
				self.transport.write(b'RFB 003.008\n')
				self._state = 1
				_q_s.logs.info(["servers",{'server':'vnc_server','action':'connection','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port}])

			def dataReceived(self, data):
				if self._state == 1:
					if data == b'RFB 003.008\n':
						self._state = 2
						self.transport.write(unhexlify('0102'))
				elif self._state == 2:
					if data == b'\x02':
						self._state = 3
						self.transport.write(_q_s.challenge)
				elif self._state == 3:
					_x = _q_s.decode(_q_s.challenge,data.hex())
					if _x:
						if _x == _q_s.password:
									_q_s.logs.info(["servers",{'server':'vnc_server','action':'login','status':'success','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':'UnKnown','password':_q_s.password}])
						else:
							_q_s.logs.info(["servers",{'server':'vnc_server','action':'login','status':'failed','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':'UnKnown','password':_x}])
					else:
						if len(data) == 16:
							#we need to check the lenth check length first
							_q_s.logs.info(["servers",{'server':'vnc_server','action':'login','status':'failed','ip':self.transport.getPeer().host,'port':self.transport.getPeer().port,'username':'UnKnown','password':data.hex()}])
					self.transport.loseConnection()
				else:
					self.transport.loseConnection()

			def connectionLost(self, reason):
				self._state = None

		factory = Factory()
		factory.protocol = CustomVNCProtocol
		reactor.listenTCP(port=self.port, factory=factory, interface=self.ip)
		reactor.run()

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'vnc_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'vnc_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'vnc_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'vnc_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'vnc_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.vnc_server_main()


	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			ip or self.ip
			port or self.port 
			username or self.username
			password or self.password
			client = vncapi.connect("{}::{}".format(self.ip,self.port),password=password)
			client.captureScreen('screenshot.png')
			client.disconnect()
		except:
			pass

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'vnc_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qvncserver = QVNCServer(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		qvncserver.run_server()


"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',category=DeprecationWarning)

from smtpd import SMTPChannel, SMTPServer
from asyncore import loop
from base64 import b64decode
from multiprocessing import Process
from psutil import process_iter
from signal import SIGTERM
from time import sleep
from smtplib import SMTP
from logging import DEBUG, basicConfig, getLogger
from pathlib import Path
from os import path
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from subprocess import Popen
from tempfile import gettempdir,_get_candidate_names
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QSMTPServer():
	def __init__(self,ip=None,port=None,username=None,password=None,mocking=False,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 25
		self.username = username or "test"
		self.password = password or "test"
		self.mocking = mocking or ''
		self.random_servers = []
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))

	def smtp_server_main(self):
		_q_s = self

		class CustomSMTPChannel(SMTPChannel):
			def smtp_EHLO(self, arg):
				_q_s.logs.info(["servers",{'server':'smtp_server','action':'connection','ip':self.addr[0],'port':self.addr[1]}])
				if not arg:
					self.push('501 Syntax: HELO hostname')
				if self._SMTPChannel__greeting:
					self.push('503 Duplicate HELO/EHLO')
				else:
					self._SMTPChannel__greeting = arg
					self.push('250-{0} Hello {1}'.format(self._SMTPChannel__fqdn, arg))
					self.push('250-8BITMIME')
					self.push('250-AUTH LOGIN PLAIN')
					self.push('250 STARTTLS')

			def smtp_AUTH(self, arg):
				try:
					if arg.startswith('PLAIN '):
						_, username, password = b64decode(arg.split(' ')[1].strip()).decode("utf-8").split('\0')
						if username ==  _q_s.username and password == _q_s.password:
							_q_s.logs.info(["servers",{'server':'smtp_server','action':'login','status':'success','ip':self.addr[0],'port':self.addr[1],'username':_q_s.username,'password':_q_s.password}])
						else:
							_q_s.logs.info(["servers",{'server':'smtp_server','action':'login','status':'faild','ip':self.addr[0],'port':self.addr[1],'username':username,'password':password}])
				except Exception as e:
					print(e)
					_q_s.logs.error(["errors",{'server':'smtp_server','error':'smtp_AUTH',"type":"error -> "+repr(e)}])

				self.push('235 Authentication successful')

			def __getattr__(self, name):
				self.smtp_QUIT(0)

		class CustomSMTPServer(SMTPServer):
			def __init__(self, localaddr, remoteaddr):
				SMTPServer.__init__(self, localaddr, remoteaddr)

			def process_message(self, peer, mailfrom, rcpttos, data,mail_options=None,rcpt_options=None):
				return

			def handle_accept(self):
				conn, addr = self.accept()
				CustomSMTPChannel(self, conn, addr)

		CustomSMTPServer((self.ip, self.port), None)
		loop(timeout=1.1,use_poll= True)

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'smtp_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
					else:
						self.logs.info(["servers",{'server':'smtp_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'smtp_server','action':'setup','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--username',str(self.username),'--password',str(self.password),'--mocking',str(self.mocking),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'smtp_server','action':'process','status':'success','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
				else:
					self.logs.info(["servers",{'server':'smtp_server','action':'process','status':'error','ip':self.ip,'port':self.port,'username':self.username,'password':self.password}])
		else:
			self.smtp_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,username=None,password=None):
		try:
			sleep(2)
			_ip = ip or self.ip
			_port = port or self.port 
			_username = username or self.username
			_password = password or self.password
			s = SMTP(_ip,_port)
			s.ehlo()
			s.login(_username,_password)
			s.sendmail("fromtest","totest","Nothing")
			s.quit()
		except:
			pass

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2) 
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'smtp_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qsmtpserver = QSMTPServer(ip=parsed.ip,port=parsed.port,username=parsed.username,password=parsed.password,mocking=parsed.mocking,logs=parsed.logs)
		qsmtpserver.run_server()



"""
//  -------------------------------------------------------------
//  author        Giga
//  project       qeeqbox/honeypots
//  email         gigaqeeq@gmail.com
//  description   app.py (CLI)
//  licensee      AGPL-3.0
//  -------------------------------------------------------------
//  contributors list qeeqbox/social-analyzer/graphs/contributors
//  -------------------------------------------------------------
"""

from warnings import filterwarnings
filterwarnings(action='ignore',module='.*OpenSSL.*')

from twisted.names import dns, error,client
from twisted.names.server import DNSServerFactory
from twisted.internet import defer, reactor
from psutil import process_iter
from signal import SIGTERM
from time import sleep
from subprocess import Popen,PIPE
from shlex import split as ssplit
from logging import DEBUG, basicConfig, getLogger
from twisted.python import log as tlog
from tempfile import gettempdir,_get_candidate_names
from socket import socket as ssocket
from socket import AF_INET,SOCK_STREAM
from pathlib import Path
from os import path
from dns.resolver import Resolver
from honeypots.helper import server_arguments, get_free_port, CustomHandler
from uuid import uuid4

class QDNSServer():
	def __init__(self,ip=None,port=None,resolver_addresses=None,logs=None):
		self.ip= ip or '0.0.0.0'
		self.port = port or 53
		self.resolver_addresses = resolver_addresses or [('8.8.8.8', 53)]
		self.process = None
		self._logs = logs
		self.setup_logger(self._logs)
		self.disable_logger()

	def disable_logger(self):
		temp_name = path.join(gettempdir(), next(_get_candidate_names()))
		tlog.startLogging(open(temp_name, 'w'), setStdout=False)

	def setup_logger(self,logs):
		self.logs = getLogger('honeypotslogger'+'_'+__class__.__name__+'_'+str(uuid4())[:8])
		self.logs.setLevel(DEBUG)
		self.logs.addHandler(CustomHandler())

	def dns_server_main(self):
		_q_s = self

		class CustomCilentResolver(client.Resolver):
			def queryUDP(self, queries, timeout=2):
				res = client.Resolver.queryUDP(self, queries, timeout)
				def queryFailed(reason):
					return defer.fail(error.DomainError())
				res.addErrback(queryFailed)
				return res

		class CustomDNSServerFactory(DNSServerFactory):
			def gotResolverResponse(self, response, protocol, message, address):
				args = (self, response, protocol, message, address)
				_q_s.logs.info(['servers',{'server':'dns_server','action':'connection','ip':address[0],'port':address[1]}])
				try:
					for items in response:
						for item in items:
							_q_s.logs.info(['servers',{'server':'dns_server','action':'query','ip':address[0],'port':address[1],'payload':item.payload}])
				except Exception as e:
					_q_s.logs.error(['errors',{'server':'dns_server','error':'gotResolverResponse','type':'error -> '+repr(e)}])
				return DNSServerFactory.gotResolverResponse(*args)


		self.resolver = CustomCilentResolver(servers=self.resolver_addresses)
		self.factory = CustomDNSServerFactory(clients=[self.resolver])
		self.protocol = dns.DNSDatagramProtocol(controller=self.factory)
		reactor.listenUDP(self.port, self.protocol,interface=self.ip)
		reactor.listenTCP(self.port, self.factory,interface=self.ip)
		reactor.run()

	def run_server(self,process=False,auto=False):
		if process:
			if auto:
				port = get_free_port()
				if port > 0:
					self.port = port
					self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--logs',str(self._logs)])
					if self.process.poll() is None:
						self.logs.info(["servers",{'server':'dns_server','action':'process','status':'success','ip':self.ip,'port':self.port}])
					else:
						self.logs.info(["servers",{'server':'dns_server','action':'process','status':'error','ip':self.ip,'port':self.port}])
				else:
					self.logs.info(["servers",{'server':'dns_server','action':'setup','status':'error','ip':self.ip,'port':self.port}])
			elif self.close_port() and self.kill_server():
				self.process = Popen(['python3',path.realpath(__file__),'--custom','--ip',str(self.ip),'--port',str(self.port),'--logs',str(self._logs)])
				if self.process.poll() is None:
					self.logs.info(["servers",{'server':'dns_server','action':'process','status':'success','ip':self.ip,'port':self.port}])
				else:
					self.logs.info(["servers",{'server':'dns_server','action':'process','status':'error','ip':self.ip,'port':self.port}])
		else:
			self.dns_server_main()

	def kill_server(self,process=False):
		try:
			self.process.kill()
			for process in process_iter():
				cmdline = ' '.join(process.cmdline())
				if '--custom' in cmdline and Path(__file__).name in cmdline:
					process.send_signal(SIGTERM)
					process.kill()
			if self.process != None:
				self.process.kill()
			return True
		except:
			pass
		return False

	def test_server(self,ip=None,port=None,domain=None):
		try:
			res = Resolver(configure=False)
			res.nameservers = [self.ip]
			res.port = self.port
			temp_domain = domain or "example.org"
			r = res.query(temp_domain, 'a')
		except:
			pass

	def close_port(self):
		sock = ssocket(AF_INET,SOCK_STREAM)
		sock.settimeout(2)
		if sock.connect_ex((self.ip,self.port)) == 0:
			for process in process_iter():
				try:
					for conn in process.connections(kind='inet'):
						if self.port == conn.laddr.port:
							process.send_signal(SIGTERM)
							process.kill()
				except:
					pass
		if sock.connect_ex((self.ip,self.port)) != 0:
			return True
		else:
			self.logs.error(['errors',{'server':'dns_server','error':'port_open','type':'Port {} still open..'.format(self.ip)}])
			return False

if __name__ == '__main__':
	parsed = server_arguments()
	if parsed.docker or parsed.aws or parsed.custom:
		qdnsserver = QDNSServer(ip=parsed.ip,port=parsed.port,resolver_addresses=parsed.resolver_addresses,logs=parsed.logs)
		qdnsserver.run_server()



from setuptools import setup, find_packages

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
	name='honeypots',
    	author='QeeqBox',
    	author_email='gigaqeeq@gmail.com',
	description="Easy to setup honeypots!",
	long_description=long_description,
	version='0.6',
	license="AGPL-3.0",
	url="https://github.com/qeeqbox/honeypots",
	packages=['honeypots'],
	include_package_data=True,
    entry_points={
        "console_scripts": [
            "honeypots = honeypots.__main__:main"
        ]
    },
	install_requires=[
		'pipenv',
		'twisted',
		'psutil',
		'psycopg2-binary',
		'dnspython',
		'requests',
		'impacket',
		'paramiko==2.7.1',
		'redis',
		'mysql-connector',
		'pycryptodome',
		'vncdotool',
		'service_identity',
		'requests[socks]',
		'pygments'
	],
	python_requires='>=3'
)



#!/usr/bin/env python

from sys import argv
from honeypots import QDNSServer, QFTPServer, QHTTPProxyServer, QHTTPServer, QHTTPSServer, QIMAPServer, QMysqlServer, QPOP3Server, QPostgresServer, QRedisServer, QSMBServer, QSMTPServer, QSOCKS5Server, QSSHServer, QTelnetServer, QVNCServer, server_arguments, clean_all
from time import sleep
from atexit import register

all_servers = ['QDNSServer','QFTPServer','QHTTPProxyServer','QHTTPServer','QHTTPSServer','QIMAPServer','QMysqlServer','QPOP3Server','QPostgresServer','QRedisServer','QSMBServer','QSMTPServer','QSOCKS5Server','QSSHServer','QTelnetServer','QVNCServer']

temp_honeypots = []

def exit_handler():
	print("Cleaning")
	clean_all()
	sleep(5)

register(exit_handler)

def list_all_honeypots():
	for honeypot in all_servers:
		print(honeypot[1:].replace('Server','').lower())

def main():
	print("\nUse enter to exit\n")
	try:
		if argv[1] == "list":
			list_all_honeypots()
		elif argv[1] == "all":
			for honeypot in all_servers:
				x = globals()[honeypot]()
				x.run_server(process=True,auto=True)
				temp_honeypots.append(x)
		else:
			servers = argv[1].split(',')
			for server in servers:
				if ":" in server:
					for honeypot in all_servers:
						if 'q{}server'.format(server.split(':')[0]).lower() == honeypot.lower():
							x = globals()[honeypot](port=int(server.split(':')[1]))
							x.run_server(process=True)
							temp_honeypots.append(x)
				else:
					for honeypot in all_servers:
						if 'q{}server'.format(server).lower() == honeypot.lower():
							x = globals()[honeypot]()
							x.run_server(process=True,auto=True)
							temp_honeypots.append(x)
		
		input("")
		for server in temp_honeypots:
			try:
				print("Killing {}".format(server.__class__.__name__))
				server.kill_server()
			except:
				pass

		if len(temp_honeypots) > 0:
			print("Please wait few seconds")
			sleep(5)
	except Exception as e:
		pass

if __name__ == "__main__":
	main()
	clean_all()